// 完毕：
using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using s2industries.ZUGFeRD;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // 此示例演示如何从 ZUGFeRD 兼容的 XML 附件中检索发票数据。
    // 所有 ZUGFeRD 数据都会打印到此示例生成的 PDF 中。
    // See ZugferdInfo 以获取类似的示例，该示例打印 ZUGFeRD 数据的选定部分。

    // 包含 ZUGFeRD 数据的示例 PDF 发票，该示例将其用作输入
    // 由ZugferdInvoice生成​​。
    // 
    // ZUGFeRD 是基于 PDF 和 XML 文件格式的德国电子发票标准。
    // 它准备改变发票的处理方式，并可供任何类型的企业使用。
    // 它将使发件人和客户的发票处理更加高效。
    // 详细信息请参见https://www.ferd-net.de/standards/what-is-zugferd/index.html。
    // 
    // 此示例​​使用 https://github.com/stephanstapel/ZUGFeRD-csharp 包
    // 解析附加到发票的 ZUGFeRD 兼容 XML。
    public class ZugferdInfoExt
    {
        public void CreatePDF(Stream stream)
        {
            // 带有 ZUGFeRD 数据的发票样本：
            var invoicePdf = Path.Combine("Resources", "PDFs", "zugferd-invoice.pdf");

            // 输出文件：
            var doc = new GcPdfDocument();
            using (FileStream fs = File.OpenRead(invoicePdf))
            {
                // 加载 ZUGFeRD 合规发票 PDF：
                var invoice = new GcPdfDocument();
                invoice.Load(fs);

                // 通过 ZUGFeRD 1.x 标准文件名从发票中获取 ZUGFeRD 附件：
                var attachment = invoice.EmbeddedFiles.Values.FirstOrDefault(it => it.File.FileName == "ZUGFeRD-invoice.xml");
                if (attachment != null)
                {
                    using (var xmlstream = attachment.GetStream())
                    {
                        // 加载发票描述符：
                        var descriptor = InvoiceDescriptor.Load(xmlstream);

                        var tl = new TextLayout(72);
                        tl.MaxWidth = doc.PageSize.Width;
                        tl.MaxHeight = doc.PageSize.Height;
                        tl.MarginAll = tl.Resolution;
                        tl.DefaultTabStops = 24;

                        // 将所有 InvoiceDescriptor 属性递归渲染到文本布局：
                        tl.AppendLine($"{nameof(InvoiceDescriptor)}:", _tfStrong);
                        RenderProperties(descriptor, tl, 1);

                        // 编写文本布局以输出 PDF
                        tl.PerformLayout(true);
                        var to = new TextSplitOptions(tl);
                        while (true)
                        {
                            var splitResult = tl.Split(to, out TextLayout rest);
                            doc.Pages.Add().Graphics.DrawTextLayout(tl, PointF.Empty);
                            if (splitResult != SplitResult.Split)
                                break;
                            tl = rest;
                        }
                        // 完毕：
                        doc.Save(stream);
                    }
                }
            }
        }

        // 递归地将所有源的属性打印到文本布局：
        private static void RenderProperties(object source, TextLayout tl, int level)
        {
            if (source == null)
                return;

            var props = source.GetType().GetProperties();
            foreach (var prop in props)
                RenderProperty(prop, source, tl, level);
        }

        // 输出的文本格式：
        private static readonly TextFormat _tfData = new TextFormat() {
            Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "segoeui.ttf")),
            FontSize = 12
        };
        private static readonly TextFormat _tfStrong = new TextFormat(_tfData) {
            Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "segoeuib.ttf"))
        };
        private static readonly TextFormat _tfLabels = new TextFormat(_tfData) {
            Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "segoeuii.ttf")),
            FontSize = 11
        };

        // 打印属性：
        private static void RenderProperty(PropertyInfo property, object parent, TextLayout tl, int level)
        {
            var space = string.Empty;
            for (int i = 0; i < level; ++i)
                space += "\t";
            var name = property.Name;
            var value = property.GetValue(parent, null);
            if (value == null)
            {
                tl.Append(space, _tfLabels);
                tl.AppendLine($"{name}:", _tfLabels);
            }
            else if (value.GetType().IsValueType || value is string)
            {
                tl.Append(space, _tfLabels);
                tl.Append($"{name}: ", _tfLabels);
                tl.AppendLine(string.Format(CultureInfo.GetCultureInfo("en-US"), "{0}", value), _tfData);
            }
            else
            {
                if (value is IEnumerable<object> collection)
                {
                    var index = 0;
                    foreach (var item in collection)
                    {
                        tl.Append(space, _tfLabels);
                        tl.AppendLine($"Collection item {name} [{index++}]:", _tfStrong);
                        RenderProperties(item, tl, level + 1);
                    }
                }
                else
                {
                    tl.Append(space, _tfLabels);
                    tl.AppendLine($"Container {name}:", _tfStrong);
                    RenderProperties(value, tl, level + 1);
                }
            }
        }
    }
}
